﻿#ifndef _OneOscilloscopeData_H_
#define _OneOscilloscopeData_H_

#include <QObject>
#include <QMap>

#include "USBInterFace.h"
#include "spdlog/spdlog.h"
#include "GlobalInfo.h"
#include "OscDataInfo.h"
#include "RingQueue/RingQueue.hpp"

#define OneOscData OneOscilloscopeData

class OneOscilloscopeData : public QObject
{
    Q_OBJECT

const uint32_t BUFFER_SIZE = 1024 * 128 ;    /* 缓冲区大小，OSCA02的缓冲区应该是128KB的SRAM */

private:
    
    OneOscilloscopeData(const OneOscilloscopeData&) = delete;
    OneOscilloscopeData& operator=(const OneOscilloscopeData&) = delete;
public:
    OneOscilloscopeData();
    ~OneOscilloscopeData();

    // static OneOscilloscopeData& getInstance()
    // {
    //     static OneOscilloscopeData instance;
    //     return instance;
    // }
    /* 初始化示波器 */
    void initOSC(int oscNum);
    /* 打开示波器 */
    bool openOSC();
    /* 关闭示波器 */
    void closeOSC();
    /* 示波器是否打开 */
    bool isOpen() { return m_isOpen; }
    /*  */
    /* 开始采集数据 */
    bool startCapture();
    /* 停止采集数据 */
    void stopCapture();

    /***** 设置示波器的功能命令 *****/
    /* 设置示波器的采样率 */
    void setSampleRate(OscSampleRate rate);
    /* 将示波器两个通道合并为一个通道 */
    void setChannelMerge(bool merge);
    /* 设置通道A输入量程 */
    void setChannelARange(OscVoltageRange range);
    /* 设置通道B输入量程 */
    void setChannelBRange(OscVoltageRange range);
    /* 设置通道耦合方式 */
    void setChannelCoupling(OscChannel channel, OscChannelCoupling coupling);
    /* 开启或关闭通道A触发 */
    void setChannelATrigger(bool enable);
    /* 开启外触发 */
    void setExternalTrigger(bool enable);
    /* 设置触发方式 */
    void setTriggerMode(OscTriggerMode mode);
    /* 设置触发电平 */
    void setTriggerLevel(unsigned char level);
    /* 设置触发灵敏度 */
    void setTriggerSensitivity(OscTriggerSensitivity sensitivity);
    /* 设置触发在缓冲区的哪个位置 */
    void setTriggerPosition(unsigned char lowByte, unsigned char highByte);
    /* 设置时间刻度 */
    void setTimeGridValue(OscTimeGridValue value);

    /* 获取示波器不同档位下的零电压值 */
    void getZeroVoltage();
    /* 打印出零电压值 */
    void printZeroVoltage(OscChannel channel);

    /* 获取不同档位下电压校准系数 */
    void getVoltageCalibration();
    /* 打印出电压校准系数 */
    void printVoltageCalibration(OscChannel channel);

    /***** 示波器其他属性 *****/
    int getOscNum() { return m_oscNum; }

private:
// public:
    /* 采集数据，这个是子线程 */
    void threadCaptureData();
    /* 处理数据线程 */
    void threadSeparateData();
    /* 处理A通道数据 */
    void threadProcessData_A();
    /* 处理B通道数据 */
    void threadProcessData_B();

    /* 解析数据，眼图需要的数据 */
    void parseEyeMapData(OscChannel chn, unsigned char* buffer, unsigned int size);
    /* 设置零电压值和电压校准系数 */
    void setZeroVoltageAndCalibration(OscChannel chn, OscVoltageRange range);
    /* 校准电压 */
    inline double calibrationVoltageA(uint8_t& data);
    inline double calibrationVoltageB(uint8_t& data);

    /* 根据示波器编号获取示波器的数据矩阵指针 */
    bool getEyeDataMatrixPtr();

private:
    std::shared_ptr<spdlog::logger> m_logger = nullptr;
    std::shared_ptr<USBInterface> m_usbInterface = nullptr;
    std::atomic_bool m_isOpen = false;                      /* 示波器是否打开 */
    std::atomic_bool m_isConnected = false;                 /* 示波器是否连接 */
    std::atomic_bool m_runCapture = false;                  /* 采集数据的线程标志 */
    std::atomic_bool m_isRunCapture = false;                /* 采集数据的线程运行标志，这个标志位作为线程运行的标志 */
    std::atomic_bool m_isCapturedData = false;              /* 是否采集到数据 */
    std::condition_variable m_condCapture;                  /* 采集数据的条件变量 */
    std::condition_variable m_condBuffer_A;                 /* 通道A的条件变量 */
    std::condition_variable m_condBuffer_B;                 /* 通道B的条件变量 */
    std::mutex m_mutexBuffer;                               /* 互斥锁，用于保护采集到的数据 */
    std::mutex m_mutexBuffer_A;                             /* 互斥锁，用于保护采集到的数据 */
    std::mutex m_mutexBuffer_B;                             /* 互斥锁，用于保护采集到的数据 */
    unsigned char* m_devBuffer = nullptr;                   /* 设备缓冲区指针，拷贝数据用的 */
    unsigned char* m_buffer = nullptr;                      /* 缓冲区指针，用于存储拷贝出来的数据 */
    unsigned char* m_bufferChnA = nullptr;                  /* 通道A的缓冲区指针 */
    unsigned char* m_bufferChnB = nullptr;                  /* 通道B的缓冲区指针 */
    // RingQueue<unsigned char*> m_ringQueue;                  /* 环形队列，用于存储示波器采集到的数据 */

    int m_oscNum = 0;                                       /* 示波器编号 */
    

    /************* 示波器控制相关 *************/
    unsigned char m_ctrlByte0 = 0;                          /* 控制字节0 */
    unsigned char m_ctrlByte1 = 0;                          /* 控制字节1 */

    QMap<OscVoltageRange, unsigned char> m_mapChAZeroVoltage;   /* 通道A的零电压值 */
    QMap<OscVoltageRange, unsigned char> m_mapChBZeroVoltage;   /* 通道B的零电压值 */
    QMap<OscVoltageRange, unsigned char> m_mapChAVoltageAmplitudeRatio;     /* 通道A的电压校准系数 */
    QMap<OscVoltageRange, unsigned char> m_mapChBVoltageAmplitudeRatio;     /* 通道B的电压校准系数 */

    unsigned char m_zeroVoltageA = 0;                       /* 通道A零电压值 */
    unsigned char m_zeroVoltageB = 0;                       /* 通道B零电压值 */
    char m_diffVoltageA = 0;                                /* 通道A零电压差值 */
    char m_diffVoltageB = 0;                                /* 通道B零电压差值 */
    double m_voltageCalibrationA = 0;                       /* 通道A电压校准系数 */
    double m_voltageCalibrationB = 0;                       /* 通道B电压校准系数 */
    double m_rangeRatioA = 0.0;                             /* 通道A量程比例 */
    double m_rangeRatioB = 0.0;                             /* 通道B量程比例 */

    std::atomic_int m_oneTimeGrid = 0;                      /* 一个时间刻度的时间长度，单位ns，和枚举值OscTimeGridValue相对应 */
    std::atomic<double> m_SampleIntervalTime = 0;           /* 采样点之间的时间间隔，单位ns（采样率的倒数，设置采样率的时候会设置） */
    // std::atomic_int m_eyeMapWidth = 0;                      /* 眼图x轴宽度（像素矩形的个数） */

    /************* 数据指针 *************/
    EyeDataMatrix* m_eyeDataMatrix_A = nullptr;               /* 眼图数据矩阵，A通道 */
    EyeDataMatrix* m_eyeDataMatrix_B = nullptr;               /* 眼图数据矩阵，B通道 */
    EyeMapMatrix* m_eyeMapMatrix_A = nullptr;                 /* 眼图矩阵，A通道 */
    EyeMapMatrix* m_eyeMapMatrix_B = nullptr;                 /* 眼图矩阵，B通道 */

};




#endif /* _OneOscilloscopeData_H_ */
